<?php
    /*
     * class Crypto
     * Klasėje yra kodavimo dekodavimo paprasti metodai.
     * encode_int() metodas verčia string eilute iš bet kokių ženklų eilute susidedančia tik iš skaičių - ASCII skaitinių atitikmenu
     *          Visi skaičiai verčiami triženklių pavydalu, tuščias vietas užpildant 0
     * decode_int() metodas verčia string eilute, susidedančia tik iš skaičių, eilute iš bet kokių ASCII ženklų
     * mbStringToArray() - metodas skaldo string tipo eilutę į masyvą
     * int_array_align_implode() - metodas  metodas verčia skaičių masyvą eilute, papildomai visus savo elementus privesdamas prie 3 ženklų pavydalo.
     * getKeyforUser() - metodas generuojantis md5 pavydalo unikalujį raktą pagal vartotojo ID, įmama iš YII, arba įvedama į metoda rankiniu budu
     */
     
    class Crypto extends STdClass {
    
        /**
         * Metodas skaldo string kintamąjį į char elementų masyvą
         * @param string $string yra įėjimo kintamasis, kuris bus išskaldytas
         * @param string $encoding yra įėjimo kintamasis, kuris žymi kodavimą. Jei neužduotas kviečiant metodą, naudojama jo default reikšmė
         * @return bool|array
         */
        public static function mbStringToArray( $string, $encoding = 'UTF-8' ) {

            $strlen = mb_strlen( $string );
        
            while ( $strlen ) {
            
                $array[] = mb_substr( $string, 0, 1, $encoding );
            
                $string = mb_substr( $string, 1, $strlen, $encoding );
            
                $strlen = mb_strlen( $string, $encoding );
            
            }
        
            if ( isset( $array ) ) {

                return ( $array );
        
            } else {

                return false;
            }
        }
    
        
        /**
         * Metodas vercia strigng simboliu masyva skaitiniu pavydalu. 
         * Kiekvienas masyvo elementas paverciamas keturių simbolių grupe. 
         * Visi rezultate gavusio masyvo elementai sujungiami į vieną eilutę
         * @param char masyvas $string_array yra įėjimo kintamasis. Visi 
         *      šio masyvo skaičiai privedami prie trijų ženklų pavydavo, tuščias vietas pildant 0, pvz 7 virsta 007.
         *      Papildomai visi masyvo elementai sujungiami į vieną string eilutę, kuri ir yra gražinamas pagametras.
         */
        public static function int_array_align_implode( $string_array ) {   

            $return_string = null;
        
            if ( isset( $string_array ) and $string_array != '' ) {
        
                foreach ( $string_array as $int_part ) {
            
                    $int_ascii = ord( $int_part );
                
                    $aligned_int_ascii = str_pad( $int_ascii, 3, '0', STR_PAD_LEFT );
                        
                    $return_string = $return_string . $aligned_int_ascii;
            
                }
            }

            return $return_string;
        }
    
        /**
         * Metodas gražina MD5 būdu užkoduotą pranešimą
         * Pusė rakto yra užduota šiame mėtode. 
         * Kita puse rakto yra vartotojo ID.
         * @param string $user_id yra user ID, naudojamas kaip rakto dalis, kuris gali būti užduotas o ne paimtas is YII, papildomai nurodžius jį kaip parametrą
         * @param string $key_string yra raktas, kuris gali būti užduotas nestandartinis, papildomai nurodžius jį kaip parametrą
         */ 
        public static function getKeyforUser( $user_id = false, $key_string = 'dzgzsūeūęū\sdvxzclaklswfdesd*fs*df' ) {
        
            //Gražinamo kintamojo generavimas
            $public_key = null;
            //Vartotojo ID gavimas
            if ( $user_id == false ) {

                $user_id = 1212;
            }
            //Rakto, susidedančio iš key_string ir user_id generavimas
            $public_key = md5( $key_string . $user_id );
            //Rakto gražinimas
            return $public_key;
        }
    
    
        /**
         * Metodas paverčia string tipo eilutę su bet kokiais char ženklais, skaičiais ar raidėmis, kita eilute, kuriuoje yra tik char skaičiai.\
         * @param string $input_string yra įėjimo kintamasis kuris verčiamas į skaičių eilutę
         * Metodas gražina šią eilutę
         */
        public static function encode_int( $input_string ) {
        
            //Gražinamo kintamojo paruošimas
            $output_int = null;
            //Kodavimas per base64
            $base64_string = base64_encode( $input_string );
            //String eilutes vertimas char masyvu
            $string_array = SELF::mbStringToArray( $base64_string );
            //Masyve esančių ženklų keitiamas ASCII skaitinėmis reikšmėmis
            $output_int = SELF::int_array_align_implode( $string_array );
            return $output_int;
         
        }
    
        /**
         * Metodas dekoduoja skaičių seką į pradinę string eilutę.
         * @param string $input_int_string yra įėjimo kintamasis kuris verčiamas iš skaičių eilutės į savo pradinį pavydalą
         * Metodas gražina dekoduotą eilutę
         */
        public static function decode_int( $input_int_string ) {
        
            //Gražinamo kintamojo paruošimas
            $output_string = null;
            //Teksto eilutes skaldymas po 3 simpbolius, dalijant ją separatoriumi |
            $text_with_separator = wordwrap( $input_int_string, 3, '|', true );
            //Eilutės su separatoriumi | skaldymas į masyvą pagal separatoriu
            $text_array = explode( '|', $text_with_separator ); 
            //Ruošiamas string kintamasis, į kuri bus talpinami masyvo duomenys
            $string_from_array = null;
            //Tikrinama ar yra masyvas ir ar tikslinga pradėti foreach ciklą
            if ( isset( $text_array ) and $text_array != '' ) {

                //Prasukamas masyvas, jo elementai verčiami iš ASCII skaičių, raidemis ir ženklais
                foreach ( $text_array as $char_int ) {

                    //Vertimas iš ASCII skaičių, ASCII simboliais
                    $char = chr( $char_int );
                    //Sujungiams masyvo elementų į vieną visumą
                    $string_from_array = $string_from_array . $char;
            
                }
            }
            //String eilutes dekodavimas iš base64
            $output_string = base64_decode( $string_from_array );
            //Rezultato gražinimas
            return $output_string;
        }
    }
